// Copyright  2007 Konesans Ltd.  For educational purposes only. Distribution in any form is strictly forbidden.

#region Using directives

using System;
using System.ComponentModel;
using System.Drawing.Design;
using System.IO;
using System.Xml;
using Microsoft.SqlServer.Dts.Runtime;
using Konesans.Dts.Runtime.SimpleTask5.Properties;

#endregion

namespace Konesans.Dts.Runtime.SimpleTask5
{
    [DtsTask(
        DisplayName = "Simple Task 5",
        Description = "A simple task with UI.",
        IconResource = "Konesans.Dts.Runtime.SimpleTask5.Square.ico", // Add icon file and set as content as Embedded Resource. Use default namepsace + filename
        TaskContact = "Simple Task; Konesans Ltd; Copyright  2007 Konesans Ltd; http://www.konesans.com",
        UITypeName = "Konesans.Dts.Runtime.SimpleTask5.SimpleTaskUI, Konesans.Dts.Runtime.SimpleTask5UI, Version=1.0.0.0, Culture=Neutral, PublicKeyToken=d8b9c3d38ec999d1")]
    public class SimpleTask5 : Task, IDTSComponentPersist
    {
        public override DTSExecResult Execute(Connections connections, VariableDispenser variableDispenser, IDTSComponentEvents componentEvents, IDTSLogging log, object transaction)
        {
            // Do something

            return DTSExecResult.Success;
        }


        #region InitializeTask
        private Connections _connections;

        public override void InitializeTask(Connections connections, VariableDispenser variableDispenser, IDTSInfoEvents events, IDTSLogging log, EventInfos eventInfos, LogEntryInfos logEntryInfos, ObjectReferenceTracker refTracker)
        {
            // Store Connections collection for use in property accessors, see ConnectionName property
            _connections = connections;
        }
        #endregion


        #region CountProperty
        private int _count;

        [Category("Task Settings"),
            DisplayName("Count Property"),
            Description("Example of an integer property for no reason.")]
        public int CountProperty
        {
            get { return _count; }
            set { _count = value; }
        }
        #endregion

        #region FileConnection
        private string _fileConnectionId;

        [Category("Task Settings"),
            Description("Example of how to reference a connection manager."),
            Editor(typeof(FileConnectionTypeEditor), typeof(UITypeEditor))]
        public string FileConnection
        {
            get { return GetConnectionName(_connections, _fileConnectionId); }
            set { _fileConnectionId = GetConnectionID(_connections, value); }
        }
        #endregion

        #region Password
        private string _password;

        [Category("Task Settings"),
            Description("Example of an write-only password property."),
            PasswordPropertyText(true)]
        public string Password
        {
            set { _password = value; }
        }
        #endregion


        #region IDTSComponentPersist Members

        public const string Property_Task = "SimpleTask3";
        public const string Property_CountProperty = "CountProperty";
        public const string Property_FileConnection = "FileConnection";
        public const string Property_Password = "Password";

        public void LoadFromXML(XmlElement node, IDTSInfoEvents infoEvents)
        {
            try
            {
                if (node.Name != Property_Task)
                    throw new Exception(String.Format("Invalid task element '{0}' in LoadFromXML.", node.Name));

                _count = Convert.ToInt32(node.Attributes.GetNamedItem(Property_CountProperty).Value);

                _fileConnectionId = node.Attributes.GetNamedItem(Property_FileConnection).Value;

                _password = node.SelectSingleNode(Property_Password).InnerText;
            }
            catch (Exception ex)
            {
                infoEvents.FireError(0, "LoadFromXML", ex.Message, "", 0);
            }
        }

        public void SaveToXML(XmlDocument doc, IDTSInfoEvents infoEvents)
        {
            try
            {
                // Create Task Element
                XmlElement taskElement = doc.CreateElement("", Property_Task, "");
                doc.AppendChild(taskElement);

                // Save CountProperty
                XmlAttribute countAttribute = doc.CreateAttribute(Property_CountProperty);
                countAttribute.Value = _count.ToString();
                taskElement.Attributes.Append(countAttribute);

                // Save FileConnection
                XmlAttribute fileAttribute = doc.CreateAttribute(Property_FileConnection);
                fileAttribute.Value = _fileConnectionId;
                taskElement.Attributes.Append(fileAttribute);


                // Save Password - SENSITIVE DATA, Write Only Property - Default persistence does not save write ony properties!

                // Create element
                XmlElement passwordElement = doc.CreateElement("", Property_Password, "");
                taskElement.AppendChild(passwordElement);
                passwordElement.InnerText = _password;

                // Add attribute to password element - Sensitive="1"
                XmlAttribute passwordSensitiveAttribute = doc.CreateAttribute("Sensitive");
                passwordSensitiveAttribute.Value = "1";
                passwordElement.Attributes.Append(passwordSensitiveAttribute);

            }
            catch (Exception ex)
            {
                infoEvents.FireError(0, "SaveXML", ex.Message, "", 0);
            }
        }

        #endregion


        #region Validate
        /// <summary>
        /// Validation of task properties.... 
        /// </summary>
        /// <param name="connections"></param>
        /// <param name="variableDispenser"></param>
        /// <param name="componentEvents"></param>
        /// <param name="log"></param>
        /// <returns></returns>
        public override DTSExecResult Validate(Connections connections, VariableDispenser variableDispenser, IDTSComponentEvents componentEvents, IDTSLogging log)
        {
            DTSExecResult result = DTSExecResult.Success;

            try
            {
                // Warn the package developer that all is not quite right, but this is not a fatal error.
                // Use to advise of normal circumstances or potential issues.
                if (_count < 10)
                {
                    componentEvents.FireWarning(0, "Source",
                                                "Warn that the count is less than 10 and this is not a good idea. Just a dummy message.",
                                                "", 0);
                }


                // Fail the validation process if you detect anything that would cause a task to fail at runtime
                if (_count <= 0)
                {
                    componentEvents.FireError(1, "Source",
                                              "Count cannot be less than or equal to zero, otherwise it breaks the task. Fix it!",
                                              "", 0);
                    result = DTSExecResult.Failure;
                }

                if (_password == null || _password.Length == 0)
                {
                    componentEvents.FireError(1, "Source",
                                              "Password cannot be null or empty. Please supply a valid password.", "",
                                              0);
                    result = DTSExecResult.Failure;
                }

                // Check file connection is valid
                if (!ValidateFileConnection(_fileConnectionId, "FileConnection", connections, componentEvents))
                {
                    result = DTSExecResult.Failure;
                }

            }
            catch (Exception ex)
            {
                componentEvents.FireError(9, "Validation Exception", ex.Message, "", 0);
                result = DTSExecResult.Failure;
            }

            return result;
        }
        #endregion

        #region ValidateFileConnection
        /// <summary>
        /// Helper method to validate a FILE connection of usage type File Exists
        /// </summary>
        /// <param name="connectionId"></param>
        /// <param name="propertyName"></param>
        /// <param name="connections"></param>
        /// <param name="componentEvents"></param>
        /// <returns></returns>
        private bool ValidateFileConnection(string connectionId, string propertyName, Connections connections, IDTSComponentEvents componentEvents)
        {
            // Check we have an input
            if (connectionId == null || connectionId.Length == 0)
            {
                componentEvents.FireError(0, "SimpleTask5", String.Format(@"No {0} value has been specified.", propertyName), "", 0);
                return false;
            }

            // Validate input type value exists in the package
            if (!connections.Contains(connectionId))
            {
                componentEvents.FireError(0, "SimpleTask5", String.Format(@"The {0} ConnectionManager '{1}' does not exist in the connections collection.", propertyName, connectionId), "", 0);
                return false;
            }
            else
            {
                string connectionName = GetConnectionName(connections, connectionId);

                #region FILE Vaidation
                if (connections[connectionId].CreationName == "FILE")
                {
                    DTSFileConnectionUsageType usageType = (DTSFileConnectionUsageType)connections[connectionId].Properties["FileUsageType"].GetValue(connections[connectionId]);
                    string path = connections[connectionId].AcquireConnection(null) as string;

                    // Check usage type is File Exists, and that it really does exist
                    if (usageType == DTSFileConnectionUsageType.FileExists)
                    {
                        if (!File.Exists(path))
                        {
                            componentEvents.FireError(0, "SimpleTask5", String.Format(Resources.ErrorFileUsageTypeFileExistsFalse, propertyName, connectionName), "", 0);
                            return false;
                        }
                    }
                    else
                    {
                        componentEvents.FireError(0, "SimpleTask5", String.Format("Invalid file connection {0}, usage type must be file exists.", connectionName), "", 0);
                    }
                }
                else
                {
                    componentEvents.FireError(0, "SimpleTask5", String.Format(Resources.ErrorInvalidConnectionType, propertyName, "FILE"), "", 0);
                    return false;
                }
                #endregion
            }

            return true;
        }
        #endregion


    }
}
